@@ -9,7 +9,7 @@ except ImportError:
     gevent_socketio_found = False
 if gevent_socketio_found:
     print('The gevent-socketio package is incompatible with this version of '
-          'the Flask-SocketIO extension. Please uinistall it, and then '
+          'the Flask-SocketIO extension. Please uninstall it, and then '
           'install the latest version of python-socketio in its place.')
     sys.exit(1)
 
@@ -44,15 +44,27 @@ class SocketIO(object):
                 isn't known at the time this class is instantiated, then call
                 ``socketio.init_app(app)`` once the application instance is
                 available.
+    :param message_queue: A connection URL for a message queue service the
+                          server can use for multi-process communication. A
+                          message queue is not required when using a single
+                          server process.
+    :param channel: The channel name, when using a message queue. If a channel
+                    isn't specified, a default channel will be used. If
+                    multiple clusters of SocketIO processes need to use the
+                    same message queue without interfering with each other, then
+                    each cluster should use a different channel.
     :param resource: The SocketIO resource name. Defaults to ``'socket.io'``.
                      Leave this as is unless you know what you are doing.
     :param kwargs: Socket.IO and Engine.IO server options.
 
     The Socket.IO server options are detailed below:
 
-    :param client_manager_class: The class that will manage the client list.
-                                 The default value is appropriate for most
-                                 cases.
+    :param client_manager: The client manager instance that will manage the
+                           client list. When this is omitted, the client list
+                           is stored in an in-memory structure, so the use of
+                           multiple connected servers is not possible. In most
+                           cases, this argument does not need to be set
+                           explicitly.
     :param logger: To enable logging set to ``True`` or pass a logger object to
                    use. To disable logging set to ``False``.
     :param binary: ``True`` to support binary payloads, ``False`` to treat all
@@ -72,7 +84,7 @@ class SocketIO(object):
                        options are "threading", "eventlet" and "gevent". If
                        this argument is not given, "eventlet" is tried first,
                        then "gevent", and finally "threading". The websocket
-                       transport is only supported in "eventlet" mode.
+                       transport is not supported in "ithreading" mode.
     :param ping_timeout: The time in seconds that the client waits for the
                          server to respond before disconnecting.
     :param ping_interval: The interval in seconds at which the client pings
@@ -103,23 +115,38 @@ class SocketIO(object):
         self.handlers = []
         self.exception_handlers = {}
         self.default_exception_handler = None
-        if app is not None:
+        if app is not None or len(kwargs) > 0:
             self.init_app(app, **kwargs)
 
     def init_app(self, app, **kwargs):
-        if not hasattr(app, 'extensions'):
-            app.extensions = {}  # pragma: no cover
-        app.extensions['socketio'] = self
+        if app is not None:
+            if not hasattr(app, 'extensions'):
+                app.extensions = {}  # pragma: no cover
+            app.extensions['socketio'] = self
         self.server_options = kwargs
 
+        if 'client_manager' not in self.server_options:
+            url = kwargs.pop('message_queue', None)
+            channel = kwargs.pop('channel', 'flask-socketio')
+            write_only = app is None
+            if url:
+                if url.startswith('redis://'):
+                    queue_class = socketio.RedisManager
+                else:
+                    queue_class = socketio.KombuManager
+                queue = queue_class(url, channel=channel,
+                                    write_only=write_only)
+                self.server_options['client_manager'] = queue
+
         resource = kwargs.pop('resource', 'socket.io')
         if resource.startswith('/'):
             resource = resource[1:]
         self.server = socketio.Server(**self.server_options)
         for handler in self.handlers:
             self.server.on(handler[0], handler[1], namespace=handler[2])
-        app.wsgi_app = _SocketIOMiddleware(self.server, app,
-                                           socketio_path=resource)
+        if app is not None:
+            app.wsgi_app = _SocketIOMiddleware(self.server, app,
+                                               socketio_path=resource)
 
     def on(self, message, namespace=None):
         """Decorator to register a SocketIO event handler.
@@ -221,11 +248,12 @@ class SocketIO(object):
     def emit(self, event, *args, **kwargs):
         """Emit a server generated SocketIO event.
 
-        This function emits a user-specific SocketIO event to one or more
-        connected clients. A JSON blob can be attached to the event as payload.
-        This function can be used outside of a SocketIO event context, so it is
-        appropriate to use when the server is the originator of an event, for
-        example as a result of a regular HTTP message. Example::
+        This function emits a SocketIO event to one or more connected clients.
+        A JSON blob can be attached to the event as payload. This function can
+        be used outside of a SocketIO event context, so it is appropriate to
+        use when the server is the originator of an event, outside of any
+        client context, such as in a regular HTTP request handler or a
+        background task. Example::
 
             @app.route('/ping')
             def ping():
@@ -318,14 +346,12 @@ class SocketIO(object):
         :param log_output: If ``True``, the server logs all incomming
                            connections. If ``False`` logging is disabled.
                            Defaults to ``True`` in debug mode, ``False``
-                           in normal mode. Unused wheb the threading async
+                           in normal mode. Unused when the threading async
                            mode is used.
-        :param kwargs: Additional web server, Socket.IO, Engine.IO options.
-                       The web server options are specific to the server used
-                       in each of the supported async modes. See the
-                       constructor of this class for the list of Socket.IO and
-                       Engine.IO options. Note that options provided here will
-                       not be available when using an external web server such
+        :param kwargs: Additional web server options. The web server options
+                       are specific to the server used in each of the supported
+                       async modes. Note that options provided here will
+                       not be seen when using an external web server such
                        as gunicorn, since this method is not called in that
                        case.
         """
@@ -342,11 +368,16 @@ class SocketIO(object):
         log_output = kwargs.pop('log_output', debug)
         use_reloader = kwargs.pop('use_reloader', debug)
         extra_files = kwargs.pop('extra_files', None)
-        self.server_options.update(kwargs)
 
         app.debug = debug
         if app.debug and self.server.eio.async_mode != 'threading':
-            app.wsgi_app = DebuggedApplication(app.wsgi_app, evalex=True)
+            # app.wsgi_app is the _SocketIOMiddleware instance, set in the
+            # constructor.
+            # app.wsgi_app.wsgi_app is the wsgi_app from the Flask application
+            # instance
+            flask_app = app.wsgi_app
+            flask_app.wsgi_app = DebuggedApplication(flask_app.wsgi_app,
+                                                     evalex=True)
 
         if self.server.eio.async_mode == 'threading':
             from werkzeug._internal import _log
@@ -358,7 +389,22 @@ class SocketIO(object):
         elif self.server.eio.async_mode == 'eventlet':
             def run_server():
                 import eventlet
-                eventlet.wsgi.server(eventlet.listen((host, port)), app,
+                eventlet_socket = eventlet.listen((host, port))
+
+                # If provided an SSL argument, use an SSL socket
+                ssl_args = ['keyfile', 'certfile', 'server_side', 'cert_reqs',
+                            'ssl_version', 'ca_certs',
+                            'do_handshake_on_connect', 'suppress_ragged_eofs',
+                            'ciphers']
+                ssl_params = {k: kwargs[k] for k in kwargs if k in ssl_args}
+                if len(ssl_params) > 0:
+                    for k in ssl_params:
+                        kwargs.pop(k)
+                    ssl_params['server_side'] = True  # Listening requires true
+                    eventlet_socket = eventlet.wrap_ssl(eventlet_socket,
+                                                        **ssl_params)
+
+                eventlet.wsgi.server(eventlet_socket, app,
                                      log_output=log_output, **kwargs)
 
             if use_reloader:
@@ -379,9 +425,10 @@ class SocketIO(object):
             if websocket:
                 self.wsgi_server = pywsgi.WSGIServer(
                     (host, port), app, handler_class=WebSocketHandler,
-                    log=log)
+                    log=log, **kwargs)
             else:
-                self.wsgi_server = pywsgi.WSGIServer((host, port), app, log=log)
+                self.wsgi_server = pywsgi.WSGIServer((host, port), app,
+                                                     log=log)
 
             if use_reloader:
                 # monkey patching is required by the reloader
@@ -423,9 +470,10 @@ class SocketIO(object):
 def emit(event, *args, **kwargs):
     """Emit a SocketIO event.
 
-    This function emits a user-specific SocketIO event to one or more connected
-    clients. A JSON blob can be attached to the event as payload. This is a
-    function that can only be called from a SocketIO event handler. Example::
+    This function emits a SocketIO event to one or more connected clients. A
+    JSON blob can be attached to the event as payload. This is a function that
+    can only be called from a SocketIO event handler, as in obtains some
+    information from the current client context. Example::
 
         @socketio.on('my event')
         def handle_my_custom_event(json):
@@ -435,13 +483,14 @@ def emit(event, *args, **kwargs):
     :param args: A dictionary with the JSON data to send as payload.
     :param namespace: The namespace under which the message is to be sent.
                       Defaults to the namespace used by the originating event.
-                      An empty string can be used to use the global namespace.
+                      A ``'/'`` can be used to explicitly specify the global
+                      namespace.
     :param callback: Callback function to invoke with the client's
                      acknowledgement.
-    :param broadcast: ``True`` to send the message to all connected clients, or
-                      ``False`` to only reply to the sender of the originating
-                      event.
-    :param room: Send the message to all the users in the given room.
+    :param broadcast: ``True`` to send the message to all clients, or ``False``
+                      to only reply to the sender of the originating event.
+    :param room: Send the message to all the users in the given room. If this
+                 argument is set, then broadcast is implied to be ``True``.
     :param include_self: ``True`` to include the sender when broadcasting or
                          addressing a room, or ``False`` to send to everyone
                          but the sender.
